#include <cstdio>
#include <unistd.h>
#include <pqxx/pqxx>
#include "glwindow.h"

class Tournament {
public:
	int id;
	std::string name;
};

Tournament active_tournament;

/* A trigger that sets a flag whenever it's trigged. */
class FlagTrigger : pqxx::trigger {
private:
	bool flag;
	
public:
	FlagTrigger(pqxx::connection_base &conn, const PGSTD::string &name)
		: pqxx::trigger(conn, name), flag(false) {}
	virtual ~FlagTrigger() throw () {}
	
	virtual void operator() (int pid)
	{
		flag = true;
		std::fprintf(stderr, "Received a flag trigger from pid %u\n", pid);
	}

	bool get_flag() const
	{
		return flag;
	}

	void reset_flag()
	{
		flag = false;
	}
};

/* A transactor that fetches the current tournament and some information about it. */
class FetchCurrentTournament : public pqxx::transactor<> {
private:
	Tournament *tourn;

public:
	FetchCurrentTournament(Tournament *tourn) : tourn(tourn) {}
	void operator() (pqxx::transaction<> &t)
	{
		pqxx::result res( t.exec("SELECT * FROM bigscreen.active_tournament NATURAL JOIN tournaments") );
		try {
			pqxx::result::tuple tournament = res.at(0);

			tourn->id = tournament["tournament"].as(tourn->id);
			tourn->name = tournament["tournamentname"].as(tourn->name);
		} catch (PGSTD::out_of_range &e) {
			tourn->id = -1;
			tourn->name = "";
		}
	}
};

void init(pqxx::connection &conn)
{
	conn.perform(FetchCurrentTournament(&active_tournament));

	if (active_tournament.id == -1) {
		std::fprintf(stderr, "No active tournament\n");
	} else {
		std::fprintf(stderr, "Current tournament is %d (name: '%s')\n",
			active_tournament.id, active_tournament.name.c_str());
	}
}

void main_loop(pqxx::connection &conn)
{
	if (active_tournament.id == -1) {
		// No active tournament, sleep a second or so and exit
		sleep(1);
		return;
	}
	
	pqxx::work t(conn, "trx");

	// fetch all songs
	pqxx::result res( t.exec("SELECT * FROM songs") );
	for (pqxx::result::const_iterator i = res.begin(); i != res.end(); ++i) {
		std::fprintf(stderr, "%s\n", i["title"].c_str());
	}
	t.commit();
	
	sleep(1);
}

int main(int argc, char **argv)
{
	GLWindow glw("CCBS bigscreen", 800, 600, 32, false, 16, -1);
	try {
		pqxx::connection conn("dbname=ccbs host=altersex.samfundet.no user=ccbs password=GeT|>>B_");
		FlagTrigger tournament_changed(conn, "active_tournament");
		
		// when active_tournament is changed, we destroy everything and start from scratch
		for ( ;; ) {
			tournament_changed.reset_flag();
			init(conn);
			do {
				main_loop(conn);
				conn.get_notifs();
			} while (!tournament_changed.get_flag());
			std::fprintf(stderr, "active_tournament changed, resetting...\n");
		}
	} catch (const std::exception &e) {
		std::fprintf(stderr, "Exception: %s\n", e.what());
		exit(1);
	}
	
	return 0;
}
